/****************************************************************************
**
** Copyright (C) 2010-2011 Iteco Trading Srl
** All rights reserved.
** Contact: Iteco Trading Srl (info@itecad.it)
**
** This file is part of the Itecad EDA software.
**
** Itecad is free software. You can redistribute it and/or modify it under the terms of
** the GNU Lesser General Public License (LGPLv2.1) as published by the Free Software Foundation
**
** This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License along with this software;
** if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
** If you have questions regarding the use of this file, please contact
** Iteco Trading Srl at info@itecad.it
**
****************************************************************************/

#include "myGraphicsScene.h"
#include "myGraphicsView.h"

extern	myGraphicsView	*GV;
extern	myGraphicsScene	*GS;

extern	win_main    *w_main;
extern	char        file_type;

extern	QPoint	drag_or_move_delta;

extern	lay_data	layer_data[LAYER_DATA_COUNT];
extern	lay_data	workspace_data[WORKSPACE_DATA_COUNT];
extern	QPen	pen_layer;
extern	QBrush	brush_layer;

extern  QList<myGraphicsItem *>     cloned_obj;
extern  QList<myComponent *>        cloned_comp;
extern  QList<node_data *>          cloned_nodes;

extern	QColor		sel_color;

extern	bool		highlight_active;

extern  bool		pcb_pad_via_filled;

myGraphicsScene::myGraphicsScene()
        : QGraphicsScene()
{
        setItemIndexMethod(QGraphicsScene::NoIndex);	//Ho sperimentato che questo algoritmo è più conveniente
}

void myGraphicsScene::clear()
{
        //Elimino tutti gli oggetti contenuti nella scena
	foreach(myGraphicsItem *obj, all_obj)
                delete obj;
        all_obj.clear();
}

void myGraphicsScene::add_item(myGraphicsItem *obj)
{
	//Inserisco l'oggetto nella giusta posizione
	for (int i = 0; i < all_obj.count(); i++)
		{
		myGraphicsItem * list_obj = all_obj.at(i);
                if (list_obj->layer_from < obj->layer_from)
			{
			all_obj.insert(i, obj);
			return;
			}
		}
	all_obj.append(obj);
}

void myGraphicsScene::remove_item(myGraphicsItem *obj)
{
	all_obj.removeAll(obj);
}

void myGraphicsScene::draw_for_screen(QPainter *painter, QMatrix *m)
{
        painter->setRenderHint(QPainter::Antialiasing, FALSE);				//Come in myGraphicsView
        painter->setRenderHint(QPainter::HighQualityAntialiasing, FALSE);		//Come in myGraphicsView
        painter->setRenderHint(QPainter::TextAntialiasing, FALSE);			//Come in myGraphicsView
        painter->setRenderHint(QPainter::SmoothPixmapTransform, FALSE);			//Come in myGraphicsView

        if (GV->is_sch && (file_type == FT_PROJECT))
                GV->draw_frame(painter, m);

        QMatrix minverted = m->inverted();
	QRect visible_rect = minverted.mapRect(painter->viewport());
	
        if (!GV->is_sch && (workspace_data[W_PCBbackground].color == QColor(0,0,0)))
                //Solo se sono nel PCB e lo sfondo è nero posso usare le trasparenze
                painter->setCompositionMode(QPainter::CompositionMode_Plus);
        else
                painter->setCompositionMode(QPainter::CompositionMode_SourceOver);

        //Gli oggetti della lista all_obj dovrebbero già essere ordinati
        //L'ordinamento avviene nella routine add_item()
        painter->setMatrixEnabled(FALSE);
	int		layer = 1234;	//E' un valore impossibile
	bool	layer_visible = TRUE;
	myGraphicsItem *obj;
	int size = all_obj.size();
	for (int i = 0; i < size; ++i)	//Uso questo sistema invece di foreach() perchè così posso usare at() che mi velocizza il tutto.
		{
		obj = all_obj.at(i);		//Nelle QList l'operatore at() è il più veloce
                if ((obj->tipo != O_PAD) && (obj->tipo != O_VIA))
                        {
                        if (layer != obj->layer_from)
                                {
                                layer_visible = GV->layer_settings[obj->layer_from].visible;
                                pen_layer.setColor(layer_data[obj->layer_from].color);
                                brush_layer.setColor(layer_data[obj->layer_from].color);
                                }
                        if (layer_visible)
                                {
                                if (!obj->is_selected)
                                    {
                                    if (visible_rect.intersects(obj->boundingRect()))
                                            {
                                            if (GV->is_sch)
                                                    obj->paint(painter, m);
                                            else
                                                    {
                                                    if (highlight_active && obj->highlight_active)
                                                        {
                                                        pen_layer.setColor(layer_data[obj->layer_from].color.lighter(150));
                                                        brush_layer.setColor(layer_data[obj->layer_from].color.lighter(150));

                                                        obj->paint(painter, m);
                                                        pen_layer.setColor(layer_data[obj->layer_from].color);
                                                        brush_layer.setColor(layer_data[obj->layer_from].color);
                                                        }
                                                    else
                                                        {
                                                        obj->paint(painter, m);
                                                        }
                                                    }
                                            }
                                    }
                                }
                        }
		}

        painter->setCompositionMode(QPainter::CompositionMode_SourceOver);

	//Disegno tutte le giunzioni dello schematico
	if (GV->is_sch)
                GV->draw_junction(painter, m);
        else
                GV->draw_pad_and_via(painter, m);
			
	pen_layer.setColor(sel_color);	
	brush_layer.setColor(sel_color);
        for (int i = 0; i < size; ++i)	//Uso questo sistema invece di foreach() perchè così posso usare at() che mi velocizza il tutto.
		{
                obj = all_obj.at(i);		//Nelle QList l'operatore at() è il più veloce
		if (obj->is_selected)
			if (visible_rect.intersects(obj->boundingRect()))
                                obj->paint(painter, m);
		}

        if ((GV->tool == T_ADD) && w_main->tool_avviato)
                {
                //Questo blocco di codice è usato anche quando si fa il paste e il duplicate
                foreach(myComponent *comp, cloned_comp)
                        {
                        size = comp->all_obj.count();
                        for (int i = 0; i < size; ++i)	//Uso questo sistema invece di foreach() perchè così posso usare at() che mi velocizza il tutto.
                                {
                                obj = comp->all_obj.at(i);		//Nelle QList l'operatore at() è il più veloce
                                        obj->paint(painter, m);
                                }
                        }
                foreach(myGraphicsItem *obj, cloned_obj)
                        obj->paint(painter, m);
                }

        painter->setMatrixEnabled(TRUE);
        /////////////////////////////////////////////////////////////
        //I drill verrano disegnati in myGraphicsView::drawForeground
        /////////////////////////////////////////////////////////////
}

void    myGraphicsScene::draw_for_printer(QPainter *painter, QMatrix *m)
{
        painter->setRenderHint(QPainter::Antialiasing, FALSE);				//Come in myGraphicsView
        painter->setRenderHint(QPainter::HighQualityAntialiasing, FALSE);		//Come in myGraphicsView
        painter->setRenderHint(QPainter::TextAntialiasing, FALSE);			//Come in myGraphicsView
        painter->setRenderHint(QPainter::SmoothPixmapTransform, FALSE);			//Come in myGraphicsView

        if (GV->is_sch && (file_type == FT_PROJECT))
                GV->draw_frame(painter, m);
        ////////////////////////////////////////////////////////////////////////////
        // Se volessi la sovrapposizione dei colori nel PCB come fatto a schermo petrei fare usare:
        //        painter->setCompositionMode(QPainter::CompositionMode_Plus);
        // come fatto in draw_for_screen() ma è brutto a stamparsi. Non piace perchè non è realistico.
        // Inoltre la sovrapposizione colori si riesce a fare solo se si lavora sulle immagini e non nella stampa pdf dove si usano i comandi grafici
        ////////////////////////////////////////////////////////////////////////////

        int	layer = 1234;	//E' un valore impossibile
        bool	layer_print = TRUE;
        myGraphicsItem	*obj;
        QList<myGraphicsItem *> all_object = GS->all_obj;
        int size = all_object.size();
        for (int i = 0; i < size; ++i)	//Uso questo sistema invece di foreach() perchè così posso usare at() che mi velocizza il tutto.
                {
                obj = all_object.at(i);		//Nelle QList l'operatore at() è il più veloce
                if ((obj->tipo != O_PAD) && (obj->tipo != O_VIA))
                        {
                        if (layer != obj->layer_from)
                                {
                                layer_print = GV->layer_settings[obj->layer_from].print;
                                pen_layer.setColor(layer_data[obj->layer_from].color);
                                brush_layer.setColor(layer_data[obj->layer_from].color);
                                }
                        if (layer_print)
                                obj->paint(painter, m);
                        }
                }

        if (GV->is_sch)
                GV->draw_junction(painter, m);
        else
                {
                QColor old_background_color = workspace_data[W_PCBbackground].color;
                workspace_data[W_PCBbackground].color = QColor(255,255,255);
                GV->draw_pad_and_via(painter, m);
                workspace_data[W_PCBbackground].color = old_background_color;
                }
}
