/****************************************************************************
**
** Copyright (C) 2010-2011 Iteco Trading Srl
** All rights reserved.
** Contact: Iteco Trading Srl (info@itecad.it)
**
** This file is part of the Itecad EDA software.
**
** Itecad is free software. You can redistribute it and/or modify it under the terms of
** the GNU Lesser General Public License (LGPLv2.1) as published by the Free Software Foundation
**
** This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License along with this software;
** if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
** If you have questions regarding the use of this file, please contact
** Iteco Trading Srl at info@itecad.it
**
****************************************************************************/

#include "win_report.h"

extern	win_main		*w_main;

win_report::win_report(QWidget *parent, Qt::WFlags tipo)
        : QDialog(parent, tipo)
{
	// Se questo QDialog viene creato con tipo == Qt::Dialog allora sono costretto ad avere il ? in alto a destra
	// Ciò forse è un bene perchè mi costringerà ad avere un minimo di help
	// Veder Qt::WindowContextHelpButtonHint per avere maggiori informazioni

	setWindowTitle(tr("Report"));	
	//setSizeGripEnabled(true);
	setModal(true);	

	QGridLayout	*layout = new QGridLayout(this);
	
	
	info.setSelectionMode(QAbstractItemView::SingleSelection);
	info.setIconSize(QSize(24,22));
	info.setFocusPolicy(Qt::NoFocus);	//Se non metto questa riga quando premo Enter la finestra non si chiude (questo perchè il fuoco non passa al comando OK)
        info.setFixedHeight(300);

	layout->addWidget(&info,0,0);
	layout->addLayout(setup_buttons(),1,0);
	layout->setSpacing(6);
        layout->setMargin(8);

	num_error = 0;
	num_warning = 0;
}

QHBoxLayout	*win_report::setup_buttons()
{
	QHBoxLayout	*L_comandi = new QHBoxLayout();
	L_comandi->setSpacing(6);
	L_comandi->setMargin(0);
	L_comandi->addWidget(&summary);
	QSpacerItem	*L_comandi_molla = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
	L_comandi->addItem(L_comandi_molla);
	QPushButton	*L_comandi_OK = new QPushButton(this);
	L_comandi_OK->setText(tr("Ok"));
	L_comandi->addWidget(L_comandi_OK);
	
	QObject::connect(L_comandi_OK, SIGNAL(clicked()), this, SLOT(accept()));

	L_comandi_OK->setAutoDefault(TRUE);
	
	return L_comandi;
}

void	win_report::insert_row(QString text, char mode)
{
	QListWidgetItem *item;
	switch (mode)
	{
	case 'W':	//Warning
		item = new QListWidgetItem(QIcon(":/Resources/warning.png"), text);
		num_warning++;
                //info.insertItem(num_warning + num_error, item);   //Inserisco al fondo
		break;
	case 'E':	//Error
		item = new QListWidgetItem(QIcon(":/Resources/error.png"), text);
		num_error++;
                //info.insertItem(0, item);   //Inserisco in prima posizione
		break;
	}
        info.addItem(item); //Inserisco tutto di seguito
}

void	win_report::finalize()
{
        if (info.count() == 0)
                {
                QListWidgetItem *item;
                item = new QListWidgetItem(QIcon(":/Resources/information.png"), tr("Netlist updated succesfully"));
                info.insertItem(0, item);
                }
        summary.setText(tr("%n Error(s)", "", num_error) + ", " + tr("%n Warning(s)", "", num_warning));
        info.scrollToBottom();   //Sposto la scroll bar al fondo
}
